==> haystack-1.0.0-dev.0.3.tar.gz: lib/src/HDictBuilder.dart <==
///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hdict_builder;

import 'HGrid.dart';
import 'HVal.dart';
import 'HStr.dart';
import 'HNum.dart';
import 'HBool.dart';
import 'HMarker.dart';
import 'UnknownNameError.dart';

/// HDictBuilder is used to construct an immutable HDict instance.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds).
class HDictBuilder {
  Map<String, HVal> _map = new Map<String, HVal>();

  HDictBuilder();

  /// Adds tag name and value, returns this.
  HDictBuilder add(String name, [Object val, String unit]) {
    if (!HDict.isTagName(name)) {
      throw new ArgumentError("Invalid tag name: $name");
    }

    if (val == null) {
      _add(name, HMarker.VAL);
    } else if (val is bool) {
      _add(name, new HBool(val));
    } else if (val is num) {
      _add(name, new HNum(val, unit));
    } else if (val is String) {
      _add(name, new HStr(val));
    } else if (val is HVal) {
      _add(name, val);
    }

    return this;
  }

  /// Adds all the name/value pairs in given HDict, returns this.
  void addDict(HDict dict) {
    for (Iterator<DictEntry> it = dict.iterator(); it.moveNext();) {
      _add(it.current.key, it.current.val);
    }
  }

  void _add(String name, HVal val) {
    _map[name] = val;
  }

  /// Converts current state to an immutable HDict instance.
  HDict toDict() {
    if (_map.isEmpty) return HDict.EMPTY;
    HDict dict = new HDict(this._map);
    this._map = new Map<String, HVal>();
    return dict;
  }

  //////////////////////////////////////////////////////////////////////////
  // Access
  //////////////////////////////////////////////////////////////////////////

  /// Returns if size is zero.
  bool isEmpty() => size == 0;

  /// Number of tag name/value pairs.
  int get size => _map.length;

  /// Returns if the given tag is present.
  bool has(String name) => get(name, false) != null;

  /// Returns if the given tag is not present.
  bool missing(String name) => get(name, false) == null;

  /// Returns a tag by name.
  ///
  /// If not found and checked if false then returns null, otherwise throw
  /// [UnknownNameException].
  HVal get(String name, [bool checked = true]) {
    HVal val = _map[name];
    if (val != null) return val;
    if (!checked) return null;
    throw new UnknownNameError(name);
  }
}

==> haystack-2.0.1.tar.gz: lib/src/HDictBuilder.dart <==
///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hdict_builder;

import 'HGrid.dart';
import 'HVal.dart';
import 'HStr.dart';
import 'HNum.dart';
import 'HBool.dart';
import 'HMarker.dart';
import 'UnknownNameError.dart';

/// HDictBuilder is used to construct an immutable HDict instance.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds).
class HDictBuilder {
  Map<String, HVal> _map = <String, HVal>{};

  HDictBuilder();

  /// Adds tag name and value, returns this.
  ///
  /// If val is null name sets with marker value.
  HDictBuilder add(String name, [Object? val, String? unit]) {
    if (!HDict.isTagName(name)) {
      throw ArgumentError('Invalid tag name: $name');
    }

    if (val == null) {
      _add(name, HMarker.VAL);
    } else if (val is bool) {
      _add(name, HBool(val));
    } else if (val is num) {
      _add(name, HNum(val, unit));
    } else if (val is String) {
      _add(name, HStr(val));
    } else if (val is HVal) {
      _add(name, val);
    }

    return this;
  }

  /// Adds all the name/value pairs in given HDict.
  ///
  /// Skips all null values.
  void addDict(HDict dict) {
    dict.forEach((key, value) {
      _add(key, value);
    });
  }

  void _add(String name, HVal val) {
    _map[name] = val;
  }

  /// Converts current state to an immutable HDict instance.
  HDict toDict() {
    if (_map.isEmpty) return HDict.EMPTY;
    final dict = HDict(_map);
    _map = <String, HVal>{};
    return dict;
  }

  //////////////////////////////////////////////////////////////////////////
  // Access
  //////////////////////////////////////////////////////////////////////////

  /// Returns if size is zero.
  bool isEmpty() => size == 0;

  /// Number of tag name/value pairs.
  int get size => _map.length;

  /// Returns if the given tag is present.
  bool has(String name) => get(name, false) != null;

  /// Returns if the given tag is not present.
  bool missing(String name) => get(name, false) == null;

  /// Returns a tag by name.
  ///
  /// If not found and checked if false then returns null, otherwise throw
  /// [UnknownNameException].
  HVal? get(String name, [bool checked = true]) {
    HVal? val = _map[name];
    if (val != null) return val;
    if (!checked) return null;
    throw UnknownNameError(name);
  }
}
